import { ListenerRssAggregator } from "listener-rss-aggregator";
import { ListenerRss } from "listener-rss";
import { PeerTubeRequester } from "../lib/peertubeRequester";

type ListenerData = ListenerRss.Config & {
  address: string;
  channelId: string;
};

let myManager: ListenerRssAggregator;
let listenersDataBinding = new Map<string, ListenerData>();
let logger: any;
let peertube: PeerTubeRequester | undefined = undefined;
let goodPeertubeCredential: boolean = false;

import * as path from "path";
import fs from "fs";

async function register({
  registerSetting,
  settingsManager,
  peertubeHelpers,
}: any) {
  const basePath = peertubeHelpers.plugin.getDataDirectoryPath();
  logger = peertubeHelpers.logger;

  registerSetting({
    name: "ytb-urls",
    label: "URL list of Youtube channel to synchronize",
    type: "input-textarea",
  });

  registerSetting({
    name: "admin-name",
    label: "Admin Username",
    type: "input",
  });

  registerSetting({
    name: "admin-password",
    label: "Admin Password",
    type: "input-password",
  });

  logger.debug("setting register");
  fs.appendFileSync(path.join(basePath, "/storage.bd"), "");

  const configAggregator = await ListenerRssAggregator.instantiateAggregator(
    path.join(basePath, "/storage.bd")
  );
  myManager = new ListenerRssAggregator(configAggregator);

  logger.debug("Aggregator created");

  const settingYtbUrls = await settingsManager.getSetting("ytb-urls");
  if (settingYtbUrls) await addListeners(settingYtbUrls);

  const settingCredentials: any = await settingsManager.getSettings([
    "admin-name",
    "admin-password",
  ]);

  if (settingCredentials["admin-name"] && settingCredentials["admin-password"])
    apiRequestInitializer({
      domainName: peertubeHelpers.config.getWebserverUrl(),
      username: settingCredentials["admin-name"],
      password: settingCredentials["admin-password"],
    });
  logger.debug("Actual config loaded");

  settingsManager.onSettingsChange(async (settings: any) => {
    if (
      !peertube ||
      peertube.username != settings["admin-name"] ||
      peertube.password != settings["admin-password"]
    )
      apiRequestInitializer({
        domainName: peertubeHelpers.config.getWebserverUrl(),
        username: settings["admin-name"],
        password: settings["admin-password"],
      });

    await addListeners(settings["ytb-urls"]);
  });

  myManager.on("newEntries", async (entries: any) => {
    const datas = listenersDataBinding.get(entries.addressListener);
    if (!datas) return;

    logger.debug(
      "New entries detected from channel #%i: %s",
      datas.channelId,
      JSON.stringify(entries)
    );
    for (const item of entries.items)
      if (peertube)
        await peertube.uploadFromUrl({
          channelId: datas.channelId,
          targetUrl: item.link,
        });
      else {
        logger.warn("Bad credential provides. New entries Skipped.");
      }
  });
}

async function apiRequestInitializer(data: PeerTubeRequester.Config) {
  peertube = new PeerTubeRequester(data);

  try {
    await peertube.requestAuthToken();
    goodPeertubeCredential = true;
    logger.debug("credential ok");
  } catch (error) {
    logger.warn("Error during the credential validation : " + error);
    peertube = undefined;
    goodPeertubeCredential = false;
  }
}

async function addListeners(listenerInput: string) {
  let listeners: ListenerData[];
  try {
    listeners = JSON.parse(listenerInput);
  } catch {
    logger.error("Malformed URL");
    return;
  }
  let newListeners = listeners.filter(
    (item) => !listenersDataBinding.has(item.address)
  );
  let removedUrls = Array.from(listenersDataBinding.keys()).filter(
    (url) => !listeners.some((listener) => listener.address === url)
  );

  for (const newItem of newListeners) {
    listenersDataBinding.set(newItem.address, newItem);
  }
  for (const removedUrl of removedUrls) {
    listenersDataBinding.delete(removedUrl);
  }

  myManager.stopAll();
  await myManager.saveOverride(listeners);

  if (logger) logger.debug("Configuration changed: " + listenerInput);

  if (goodPeertubeCredential) myManager.startAll();
}

async function unregister() {
  myManager.stopAll();
}

module.exports = {
  register,
  unregister,
};
